//*****************************************************************************
//
// Datei       : bitvecto.h
// Autor       : Markus Halla
//
// Funktion    : Definiert eine Klasse fr eine Mengenreprsentation mit einem
//               Bit-Vektor.
//
//*****************************************************************************

#ifndef __BITVECTOR_H
#define __BITVECTOR_H

#include <stdio.h>
#include "const.h"

//*****************************************************************************
//
// Klasse      : BitVector
// Autor       : Markus Halla
//
// Funktion    : Mengen-Klasse mit einem Bit-Vektor.
//
//*****************************************************************************

class BitVector {
protected:
    SetElement size;       // Die Anzahl der mglichen Elemente in der Menge
    unsigned char* data;   // Der Speicherbereich der Menge
    SetElement dataSize;   // Die Gre des Speicherbereichs der Menge
    SetElement pos;        // Die Position des Iterators auf der Menge
    int posInit;           // Der Status des Iterators

    // Die Anzahl von Bits in einem Datenelement
    const bitsInElement = sizeof(unsigned char) * 8;

    // Iteratorzustand
    const IteratorStart = 0;
    const IteratorWork  = 1;
    const IteratorEnd   = 2;

public:
    // Fehlerklassen
    class E_Size {};     // Unterschiedliche Mengengren

    // Konstruktor
    BitVector(const SetElement newSize);
    // Copy-Konstruktor
    BitVector(const BitVector& rValue);
    // Destruktor
    ~BitVector();
    // Zuweisungsoperator
    const BitVector& operator=(const BitVector& rValue);

    // Liefert die Anzahl der mglichen Elemente der Menge
    SetElement getSize() const;

    // Gehrt das Element mit der Nummer "nr" zur Menge?
    bool member(const SetElement nr) const;

    // Fge das Element mit der Nummer "nr" zur Menge hinzu
    void addMember(const SetElement nr);

    // Entferne das Element mit der Nummer "nr" aus der Menge
    void deleteMember(const SetElement nr);

    // Lsche alle Elemente aus der Menge
    void clearSet();

    // Liefert die Nummer eines beliebigen Elements das zur Menge gehrt
    SetElement chooseOne() const;

    // Liefert die Anzahl der Elemente in der Menge
    SetElement cardinality() const;

    // Vergleicht die eigene mit der "rValue" Menge
    bool operator==(const BitVector& rValue) const;
    bool operator!=(const BitVector& rValue) const;

    // Liefert die Vereinigung der eigenen und der "rValue" Menge
    BitVector& operator|(const BitVector& rValue) const;

    // Wandelt die eigene Menge in die Vereinigung der beiden Mengen um
    void vereinigung(const BitVector& rValue1, const BitVector& rValue2);

    // Liefert den Durchschnitt der eigenen und der "rValue" Menge
    BitVector& operator&(const BitVector& rValue) const;

    // Wandelt die eigene Menge in den Durchschnitt der beiden Mengen um
    void intersect(const BitVector& rValue1, const BitVector& rValue2);

    // Differenz zwischen der eigenen und der "rValue" Menge
    BitVector& operator^(const BitVector& rValue) const;

    // Wandelt die eigene Menge in die Differenz der beiden Mengen um
    void difference(const BitVector& rValue1, const BitVector& rValue2);

    // Liefert das Komplement der eigenen Menge
    BitVector& operator!() const;

    // Wandelt die eigene Menge in das Komplement der "rValue" Menge um
    void complement(const BitVector& rValue);

    // Setzt den Iterator an den Anfang der Menge
    void setStart();

    // Liefert die aktuelle Elementnummer zurck und setzt den Iterator weiter
    SetElement getNext();

    // Gibt die Elemente der Menge aus - Testfunktion
    void print(FILE *fp);
};

//*****************************************************************************
//                                      Definiton der Inline-Element-Funktionen
//*****************************************************************************

//*****************************************************************************
// Klasse      : BitVector
// Methode     : getSize
//
// Funktion    : Liefert die Anzahl der mglichen Elemente der Menge
// Parameter   : ---
// Ergebnis    : Anzahl der mglichen Elemente der Menge
//
//*****************************************************************************
inline SetElement BitVector::getSize() const
{
    return size;
};

//*****************************************************************************
// Klasse      : BitVector
// Methode     : member
//
// Funktion    : Stellt fest ob das Element in der Menge enthalten ist
// Parameter   : nr - Nummer des gesuchten Elements
// Ergebnis    : Boolscher Wert - Testergebnis
//
//*****************************************************************************
inline bool BitVector::member(const SetElement nr) const
{
    // Existieren berhaupt soviele Elemente?
    if (size <= nr)
        return false;

    // Das Bit berprfen
    return ((data[nr / bitsInElement] & (1 << (nr % bitsInElement))) != 0);
};

//*****************************************************************************
// Klasse      : BitVector
// Methode     : addMember
//
// Funktion    : Fgt das Element in die Menge hinzu
// Parameter   : nr - Nummer des gesuchten Elements
// Ergebnis    : ---
//
//*****************************************************************************
inline void BitVector::addMember(const SetElement nr)
{
    // Existieren berhaupt soviele Elemente?
    if (size > nr)
        // Das Bit setzen
        data[nr / bitsInElement] |= (1 << (nr % bitsInElement));
};

//*****************************************************************************
// Klasse      : BitVector
// Methode     : deleteMember
//
// Funktion    : Entfernt das Element in die Menge hinzu
// Parameter   : nr - Nummer des gesuchten Elements
// Ergebnis    : ---
//
//*****************************************************************************
inline void BitVector::deleteMember(const SetElement nr)
{
    // Existieren berhaupt soviele Elemente?
    if (size > nr)
        // Das Bit lschen
        data[nr / bitsInElement] &= ~(1 << (nr % bitsInElement));
};

//*****************************************************************************
// Klasse      : BitVector
// Methode     : operator!= - Vergleichsoperator
//
// Funktion    : Vergleicht die beiden Menge
// Parameter   : rValue - Referenz auf die zu vergleichende Menge
// Ergebnis    : Boolscher Wert - Vergleichsergebnis
//
//*****************************************************************************
inline bool BitVector::operator!=(const BitVector& rValue) const
{
    return !operator==(rValue);
}

#endif

